@@ -12,7 +12,8 @@
 #'
 #' @param desc test name.  Names should be kept as brief as possible, as they
 #'   are often used as line prefixes.
-#' @param code test code containing expectations
+#' @param code test code containing expectations.  Braces (`{}`) should always be
+#'   used in order to get accurate location data for test failures.
 #' @export
 #' @examples
 #' test_that("trigonometric functions match identities", {
@@ -27,40 +28,61 @@
 #' })
 #' }
 test_that <- function(desc, code) {
+  if (!is.character(desc) || length(desc) != 1) {
+    abort("`desc` must be a string")
+  }
+
+  reporter <- get_reporter()
+  if (is.null(reporter)) {
+    reporter <- local_interactive_reporter()
+  }
+
+  local_test_context()
+
   code <- substitute(code)
-  test_code(desc, code, env = parent.frame())
+
+  if (edition_get() >= 3) {
+    if (!is_call(code, "{")) {
+      warn("The `code` argument to `test_that()` must be a braced expression to get accurate file-line information for failures.")
+    }
+  }
+
+  test_code(desc, code, env = parent.frame(), reporter = reporter)
 }
 
-test_code <- function(test, code, env = test_env(), skip_on_empty = TRUE) {
-  if (!is.null(test)) {
-    get_reporter()$start_test(context = get_reporter()$.context, test = test)
-    on.exit(get_reporter()$end_test(context = get_reporter()$.context, test = test))
+# Access error fields with `[[` rather than `$` because the
+# `$.Throwable` from the rJava package throws with unknown fields
+test_code <- function(test, code, env = test_env(), reporter = get_reporter(), skip_on_empty = TRUE) {
+  reporter <- reporter %||% StopReporter$new()
+  if (!is.null(test) && !is.null(reporter)) {
+    reporter$start_test(context = reporter$.context, test = test)
+    on.exit(reporter$end_test(context = reporter$.context, test = test))
   }
 
   ok <- TRUE
-  register_expectation <- function(e) {
-    calls <- e$expectation_calls
-    srcref <- find_first_srcref(calls)
 
+  # @param debug_end How many frames should be skipped to find the
+  #   last relevant frame call. Only useful for the DebugReporter.
+  register_expectation <- function(e, debug_end) {
+    # Find test environment on the stack
+    start <- eval_bare(quote(base::sys.nframe()), test_env) + 1L
+
+    srcref <- e[["srcref"]] %||% find_first_srcref(start)
     e <- as.expectation(e, srcref = srcref)
-    e$call <- calls
-    e$start_frame <- attr(calls, "start_frame")
-    e$end_frame <- e$start_frame + length(calls) - 1L
-    e$test <- test %||% "(unknown)"
+
+    # Data for the DebugReporter
+    if (debug_end >= 0) {
+      e$start_frame <- start
+      e$end_frame <- sys.nframe() - debug_end - 1L
+    }
+
+    e$test <- test %||% "(code run outside of `test_that()`)"
+
     ok <<- ok && expectation_ok(e)
-    get_reporter()$add_result(context = get_reporter()$.context, test = test, result = e)
+    reporter$add_result(context = reporter$.context, test = test, result = e)
   }
 
   frame <- sys.nframe()
-  frame_calls <- function(start_offset, end_offset, start_frame = frame) {
-    sys_calls <- sys.calls()
-    start_frame <- start_frame + start_offset
-    structure(
-      sys_calls[(start_frame):(length(sys_calls) - end_offset - 1)],
-      start_frame = start_frame
-    )
-  }
-
   # Any error will be assigned to this variable first
   # In case of stack overflow, no further processing (not even a call to
   # signalCondition() ) might be possible
@@ -85,18 +107,18 @@ test_code <- function(test, code, env = test_env(), skip_on_empty = TRUE) {
     options(expressions = expressions_opt_new)
     on.exit(options(expressions = expressions_opt), add = TRUE)
 
-    # Capture call stack, removing last calls from end (added by
-    # withCallingHandlers), and first calls from start (added by
-    # tryCatch etc).
-    e$expectation_calls <- frame_calls(11, 2)
+    # Add structured backtrace to the expectation
+    if (can_entrace(e)) {
+      e <- cnd_entrace(e)
+    }
 
     test_error <<- e
 
     # Error will be handled by handle_fatal() if this fails; need to do it here
     # to be able to debug with the DebugReporter
-    register_expectation(e)
+    register_expectation(e, 2)
 
-    e$handled <- TRUE
+    e[["handled"]] <- TRUE
     test_error <<- e
   }
   handle_fatal <- function(e) {
@@ -104,48 +126,66 @@ test_code <- function(test, code, env = test_env(), skip_on_empty = TRUE) {
     # Error caught in handle_error() has precedence
     if (!is.null(test_error)) {
       e <- test_error
-      if (isTRUE(e$handled)) {
+      if (isTRUE(e[["handled"]])) {
         return()
       }
     }
 
-    if (is.null(e$expectation_calls)) {
-      e$expectation_calls <- frame_calls(0, 0)
-    }
-
-    register_expectation(e)
+    register_expectation(e, 0)
   }
   handle_expectation <- function(e) {
     handled <<- TRUE
-    e$expectation_calls <- frame_calls(11, 6)
-    register_expectation(e)
+    register_expectation(e, 6)
     invokeRestart("continue_test")
   }
   handle_warning <- function(e) {
+    # When options(warn) < 0, warnings are expected to be ignored.
+    if (getOption("warn") < 0) {
+      handled <<- TRUE
+      return()
+    }
+
+    # When options(warn) >= 2, warnings are converted to errors.
+    # So, do not handle it here so that it will be handled by handle_error.
+    if (getOption("warn") >= 2) {
+      return()
+    }
+
+    if (!inherits(e, "testthat_warn")) {
+      e <- cnd_entrace(e)
+    }
+
     handled <<- TRUE
-    e$expectation_calls <- frame_calls(11, 5)
-    register_expectation(e)
-    invokeRestart("muffleWarning")
+    register_expectation(e, 5)
+
+    maybe_restart("muffleWarning")
   }
   handle_message <- function(e) {
     handled <<- TRUE
-    invokeRestart("muffleMessage")
+    if (edition_get() < 3) {
+     maybe_restart("muffleMessage")
+    }
   }
   handle_skip <- function(e) {
     handled <<- TRUE
 
     if (inherits(e, "skip_empty")) {
-      # Need to generate call as if from test_that
-      e$expectation_calls <- frame_calls(0, 12, frame - 1)
+      # If we get here, `code` has already finished its evaluation.
+      # Find the srcref in the `test_that()` frame above.
+      e$srcref <- find_first_srcref(frame - 1)
+      debug_end <- -1
     } else {
-      e$expectation_calls <- frame_calls(11, 2)
+      debug_end <- 2
     }
 
-    register_expectation(e)
+    register_expectation(e, debug_end)
     signalCondition(e)
   }
 
   test_env <- new.env(parent = env)
+  old <- options(rlang_trace_top_env = test_env)[[1]]
+  on.exit(options(rlang_trace_top_env = old), add = TRUE)
+
   tryCatch(
     withCallingHandlers(
       {
@@ -169,31 +209,3 @@ test_code <- function(test, code, env = test_env(), skip_on_empty = TRUE) {
 
   invisible(ok)
 }
-
-#' R package to make testing fun!
-#'
-#' Try the example below. Have a look at the references and learn more
-#' from function documentation such as [expect_that()].
-#'
-#' @section Options:
-#' - `testthat.use_colours`: Should the output be coloured? (Default: `TRUE`).
-#' - `testthat.summary.max_reports`: The maximum number of detailed test
-#'    reports printed for the summary reporter (default: 10).
-#' - `testthat.summary.omit_dots`: Omit progress dots in the summary reporter
-#'    (default: `FALSE`).
-#'
-#' @import rlang
-#' @keywords internal
-#' @useDynLib testthat, .registration = TRUE
-#' @references Wickham, H (2011). testthat: Get Started with Testing.
-#' \strong{The R Journal} \emph{3/1} 5-10.
-#' \url{https://journal.r-project.org/archive/2011-1/RJournal_2011-1_Wickham.pdf}
-#'
-#' \url{http://adv-r.had.co.nz/Testing.html}
-#'
-#' @examples
-#' library(testthat)
-#' a <- 9
-#' expect_that(a, is_less_than(10))
-#' expect_lt(a, 10)
-"_PACKAGE"
